------------------------------------------------------------------------------
--						Caches all spells the user knows					--
------------------------------------------------------------------------------
local SpellCache =  FlexBar2:NewModule("SpellCache", "LibRockEvent-1.0");
-- If this var is incremented, SetDefaults() will be called for this module thus resetting all its settings. Though there are no settings for this module, it has to be declared
SpellCache.Version = 1;

local Cache = {};
SpellCache.Cache = Cache;

function SpellCache:OnEnable()
	-- Schedule a cache update on initial loading
    SpellCache.ScheduleUpdate = true;
	-- Schedule a cache update when new spells are learned
    self:AddEventListener("LEARNED_SPELL_IN_TAB", function() SpellCache.ScheduleUpdate = true; self:DispatchEvent("SpellCacheUpdate"); end);
end

function SpellCache:BuildSpellCache()
    FlexBar2:Debug("Building SpellCache");
	-- Clear the old cache first
    for SpellName, _ in pairs(Cache) do 
        Cache[SpellName] = nil;
    end
	for TabId = 1, GetNumSpellTabs() do
		local TabName, _, TabOffset, TabNumSpells = GetSpellTabInfo(TabId);  
		for SpellSlot = TabOffset + 1, TabOffset + TabNumSpells do
			local SpellName, SpellRank = GetSpellName(SpellSlot, BOOKTYPE_SPELL);
			Cache[SpellName] = SpellSlot;
			Cache[SpellName .. "()"] = SpellSlot;
			Cache[SpellName .. "(" .. SpellRank .. ")"] = SpellSlot;
		end
	end
end

-- Small function to get something from the cache easely
function SpellCache:GetSpellInfo(SpellName)
	-- If an update is scheduled, update before returning stuff
	if(SpellCache.ScheduleUpdate == true) then
		SpellCache:BuildSpellCache();
		SpellCache.ScheduleUpdate = false;
	end
    return Cache[SpellName];
end

